@@ -23,7 +23,7 @@ $.select = function(selector) {
 };
 
 $.htmlEncode = function(value) {
-	return $('<div>').text(value).html();
+	return $('<div>').text(value).html().replace(/&nbsp;/g, '\xa0');
 };
 
 $.htmlDecode = function(value) {
@@ -32,8 +32,23 @@ $.htmlDecode = function(value) {
 
 // Create jQuery DOM nodes from arbitrary text contents.
 // Do not use $(contents) as $() is supposed to accept only top-level tags or jQuery selectors, not arbitrary text.
-$.contents = function(contents) {
-    return $('<span>').html(contents).contents();
+$.contents = function(contents, initCustomElements) {
+    var $contents = $('<span>').html(contents).contents();
+    if (initCustomElements) {
+        // https://stackoverflow.com/questions/55791168/initialisation-of-custom-elements-inside-document-fragment
+        // Append fragment to document so the custom elements are properly initialized,
+        // otherwise nested components test will fail):
+        /*
+        var $span = $('span').css({'display': 'none'});
+        $span.appendTo('body');
+        $contents.appendTo($span);
+        $contents.detach();
+        $span.remove();
+        */
+        $contents.appendTo('body');
+        $contents.detach();
+    }
+    return $contents;
 };
 
 $.parseUrl = function(url) {
@@ -589,86 +604,6 @@ $.fn.linkPreview = function(method) {
 
 };
 
-/**
- * Change properties of Bootstrap popover.
- */
-$.fn.changePopover = function(opts) {
-    return this.each(function() {
-        for (var opt in opts) {
-            if (opts.hasOwnProperty(opt)) {
-                $(this).data('bs.popover').options[opt] = opts[opt];
-            }
-        }
-    });
-};
-
-
-/**
- * Bootstrap popover notification.
- * Changes properties of Bootstrap popover, show popover and move window scrollpos to related location hash.
- */
-$.fn.toPopover = function(opts) {
-    return this.each(function() {
-        var $this = $(this);
-        $this
-        .changePopover(opts)
-        .popover('show');
-        window.location.hash = '#' + $this.prop('name');
-    });
-};
-
-
-/**
- * Get tip DOM elements for each selected popover.
- */
-$.fn.getPopoverTip = function() {
-    var result = [];
-    this.each(function() {
-        var data = $(this).data();
-        if (typeof data['bs.popover'] === 'object' &&
-                typeof data['bs.popover'].tip === 'function') {
-            // Bootstrap 3
-            var tip = data['bs.popover'].tip();
-        } else {
-            // Bootstrap 4
-            var tip = $.id(
-                $(this).attr('aria-describedby')
-            );
-        }
-        if (tip.length > 0) {
-            result.push(tip.get(0));
-        }
-    });
-    return $(result);
-};
-
-
-/**
- * Checks whether Boostrap popover bound to selected element(s) is visible.
- */
-$.fn.getVisiblePopovers = function() {
-    var result = []
-    this.each(function() {
-        // Boostrap 3 uses '.in', Bootstrap 4 uses '.show'
-        if ($(this).getPopoverTip().filter('.in, .show').length > 0) {
-            result.push(this);
-        }
-    });
-    return $(result);
-};
-
-
-$.fn.closeVisiblePopovers = function() {
-    this.find('[data-toggle="popover"]').getVisiblePopovers().each(function() {
-        var evt = $(this).data('trigger');
-        if (evt !== undefined) {
-            $(this).trigger(evt);
-        } else {
-            $(this).popover('hide');
-        }
-    });
-};
-
 
 /**
  * Infinite virtual scroller plugin.
