@@ -173,10 +173,10 @@ class ActionsView(FormatTitleMixin, ViewmodelView):
         self.kwargs = kwargs
         result = self.get_action_handler()()
         if result is None:
-            # Will process client-side App.Actions class "callback_{viewmodel_name}"  method.
+            # Will process client-side Actions class "callback_{viewmodel_name}"  method.
             result = vm_list(view=self.viewmodel_name)
         elif result is False:
-            # Will suppress client-side App.Actions class "callback_{viewmodel_name}" method.
+            # Will suppress client-side Actions class "callback_{viewmodel_name}" method.
             result = vm_list()
         elif not isinstance(result, list):
             result = vm_list(result)
@@ -211,18 +211,18 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
     def get_default_action_name(self):
         return 'edit_inline' if self.form is None else 'edit_form'
 
-    def __init__(self, **kwargs):
+    def setup(self, request, *args, **kwargs):
         if self.model is None:
             if self.form is not None:
                 self.model = self.form._meta.model
             elif self.form_with_inline_formsets is not None:
-                form_class = self.form_with_inline_formsets(self.request).get_form_class()
+                form_class = self.form_with_inline_formsets(request).get_form_class()
                 if form_class is None:
                     raise ValueError('Neither a model class attribute nor form_with_inline_formsets defines model class')
                 self.model = form_class._meta.model
             else:
                 raise ValueError('model class attribute is undefined')
-        super().__init__(**kwargs)
+        super().setup(request, *args, **kwargs)
 
     def get_actions(self):
         return {
@@ -301,7 +301,7 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
 
     # KoGridView uses .request_get() value because one grid may edit different model objects.
     # ModelFormActionsView may use either pk_url_kwarg value or .request_get() value.
-    # See also App.EditForm.init() / .runComponent() at client-side.
+    # See also EditForm.init() / .runComponent() at client-side.
     def get_pk_val(self):
         return self.request_get('pk_val') if self.pk_url_kwarg is None else self.kwargs[self.pk_url_kwarg]
 
@@ -345,7 +345,10 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
         if callable(getattr(self, handler_name, None)):
             getattr(self, handler_name)(**kwargs)
 
-    def vm_form(self, form, template=None, verbose_name=None, form_action='save_form', action_query: dict = None):
+    def vm_form(
+            self, form, template=None, verbose_name=None, form_action='save_form',
+            action_query: dict = None, callback_action=None
+    ):
         if template is None:
             template = self.form_template
         if action_query is None:
@@ -359,7 +362,7 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
         })
         if verbose_name is None:
             verbose_name = get_verbose_name(form.Meta.model)
-        return vm_list({
+        vm = {
             'last_action': form_action,
             'title': format_html(
                 '{}: {}',
@@ -367,9 +370,15 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
                 verbose_name
             ),
             'message': form_html
-        })
+        }
+        if callback_action is not None:
+            vm['callback_action'] = callback_action
+        return vm_list(vm)
 
-    def vm_inline(self, ff, template=None, verbose_name=None, form_action='save_inline', action_query: dict = None):
+    def vm_inline(
+            self, ff, template=None, verbose_name=None, form_action='save_inline',
+            action_query: dict = None, callback_action=None
+    ):
         if template is None:
             template = self.inline_template
         if action_query is None:
@@ -384,7 +393,7 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
         })
         if verbose_name is None:
             verbose_name = get_verbose_name(ff.get_form_class().Meta.model)
-        return vm_list({
+        vm = {
             'last_action': form_action,
             'title': format_html(
                 '{}: {}',
@@ -392,7 +401,10 @@ class ModelFormActionsView(ActionsView, FormViewmodelsMixin):
                 verbose_name
             ),
             'message': ff_html
-        })
+        }
+        if callback_action is not None:
+            vm['callback_action'] = callback_action
+        return vm_list(vm)
 
     def get_initial(self):
         return self.initial.copy()
@@ -548,8 +560,12 @@ class GridActionsMixin(ModelFormActionsView):
     # Set to dict with key fieldname: value related_model to use prefixed field names,
     # Set to list of two-element tuples to use duplicate prefixed field names for related models (eg. generic relationships).
     # Set to list of related models to use non-prefixed root field names.
-    # See also .get_related_model_fields_verbose_names() and App.ko.GridColumnOrder.renderRowValue() implementations.
+    # See also .get_related_model_fields_verbose_names() and GridColumnOrder.renderRowValue() implementations.
     related_models = None
+    virtual_fields = None
+
+    def setup(self, request, *args, **kwargs):
+        super().setup(request, *args, **kwargs)
 
     def get_actions(self):
         return {
@@ -590,7 +606,7 @@ class GridActionsMixin(ModelFormActionsView):
             'button_footer': OrderedDict(),
             'pagination': OrderedDict([
                 ('rows_per_page', {
-                    'classPath': 'App.ko.RowsPerPageAction',
+                    'classPath': 'GridRowsPerPageAction',
                     'localName': _('Rows per page'),
                     'css': {
                         'iconui': 'iconui-th-list'
@@ -731,22 +747,32 @@ class GridActionsMixin(ModelFormActionsView):
         if not isinstance(grid_fields, list):
             self.vm_error('grid_fields must be list')
         for field_def in grid_fields:
-            if isinstance(field_def, tuple):
-                vm_grid_fields.append({
+            vm_field = None
+            if isinstance(field_def, dict):
+                vm_field = {
+                    'field': field_def['field'],
+                    'name': field_def['name'] if 'name' in field_def else self.get_field_verbose_name(field_def['field']),
+                }
+            elif isinstance(field_def, tuple):
+                vm_field = {
                     'field': field_def[0],
                     'name': field_def[1]
-                })
+                }
             elif isinstance(field_def, str):
-                vm_grid_fields.append({
+                vm_field = {
                     'field': field_def,
                     'name': self.get_field_verbose_name(field_def)
-                })
-            elif isinstance(field_def, list):
-                vm_compound_fields = []
-                self.vm_add_grid_fields(field_def, vm_compound_fields)
-                vm_grid_fields.append(vm_compound_fields)
+                }
+            if vm_field is None:
+                if isinstance(field_def, list):
+                    vm_compound_fields = []
+                    self.vm_add_grid_fields(field_def, vm_compound_fields)
+                    vm_grid_fields.append(vm_compound_fields)
+                else:
+                    self.vm_error('grid_fields list values must be instances of str / tuple / list / dict')
             else:
-                self.vm_error('grid_fields list values must be instances of str or tuple or list')
+                vm_grid_fields.append(vm_field)
+                self.field_names[vm_field['field']] = vm_field['name']
 
     def vm_get_grid_fields(self):
         vm_grid_fields = []
@@ -754,13 +780,13 @@ class GridActionsMixin(ModelFormActionsView):
         return vm_grid_fields
 
     # Collect field names verbose_name or i18n of field names from related model classes when available.
-    # These usually are stored into App.ko.Grid.meta.fkNestedListOptions.i18n
+    # These usually are stored into Grid.meta.fkNestedListOptions.i18n
     def get_related_model_fields_verbose_names(self, add_field_related=None):
         # Grid model fields are not rendered as nested fields in grid, thus are not included into result of this call.
         related_verbose_names = {}
         related_models = self.get_related_models(add_field_related=add_field_related)
         # See the description of related_models class attribute.
-        # The value of field_name will be used at client-side as App.renderNestedList() options.keyPrefix attribute.
+        # The value of field_name will be used at client-side as renderNestedList() options.keyPrefix attribute.
         # See grid.js for more details.
         for field_name, model in sdv.iter_enumerate(related_models, repeated_keys=True):
             verbose_names = self.get_model_fields_verbose_names(field_name, model)
@@ -791,7 +817,7 @@ class GridActionsMixin(ModelFormActionsView):
             i18n = self.get_related_model_fields_verbose_names(add_field_related=True)
             if len(i18n) > 0:
                 meta['fkNestedListOptions']['i18n'] = i18n
-            # Current model verbose / local field names used by client-side App.FkGridWidget.setDisplayValue() call.
+            # Current model verbose / local field names used by client-side FkGridWidget.setDisplayValue() call.
             meta['listOptions'] = {
                 'showKeys': True,
                 'i18n': self.get_model_fields_verbose_names()
@@ -800,12 +826,14 @@ class GridActionsMixin(ModelFormActionsView):
         return meta
 
     def action_meta(self):
+        # self.get_filters may fail in case self.vm_get_grid_fields() is not called first.
+        vm_grid_fields = self.vm_get_grid_fields()
         vm = {
             'action_kwarg': self.action_kwarg,
             'sortOrders': self.allowed_sort_orders,
             'meta': self.get_ko_meta(),
             'actions': self.vm_get_actions(),
-            'gridFields': self.vm_get_grid_fields(),
+            'gridFields': vm_grid_fields,
             'filters': self.get_filters()
         }
         if self.__class__.mark_safe_fields is not None:
@@ -841,10 +869,10 @@ class GridActionsMixin(ModelFormActionsView):
 #     UrlPath(MyModelGrid)(name='my_model_grid')
 # To browse specified Django model rows and columns.
 #
-# HTTP GET response is 'Content-Type: text/html' generated from template_name, which should have App.ko.Grid client-side
+# HTTP GET response is 'Content-Type: text/html' generated from template_name, which should have Grid client-side
 # component defined. See 'cbv_grid.htm' for example of component definition. It is optional and is not required.
 #
-# HTTP POST response is AJAX JSON for App.ko.Grid / App.FkGridWidget Javascript components.
+# HTTP POST response is AJAX JSON for Grid / FkGridWidget Javascript components.
 #
 class KoGridView(BaseFilterView, GridActionsMixin):
 
@@ -920,7 +948,7 @@ class KoGridView(BaseFilterView, GridActionsMixin):
             for filter_field, filter_def in view_allowed_filter_fields.items():
                 if isinstance(filter_def, dict) and 'pageRoute' in filter_def:
                     # 'type': 'fk' filter field with 'pageRoute' autodiscovery.
-                    related_view = tpl.resolve_grid(
+                    related_view_cls, related_view_kwargs = tpl.resolve_grid(
                         request,
                         view_options=filter_def
                     )
@@ -930,7 +958,8 @@ class KoGridView(BaseFilterView, GridActionsMixin):
                     if 'type' in field_fkGridOptions:
                         del field_fkGridOptions['type']
                     # Apply relations to fkGridOptions recursively.
-                    field_fkGridOptions.update(related_view().discover_grid_options(request))
+                    related_view = related_view_cls(**related_view_kwargs)
+                    field_fkGridOptions.update(related_view.discover_grid_options(request))
                     grid_options['fkGridOptions'][filter_field] = field_fkGridOptions
             if self.preload_meta_list:
                 grid_options['preloadedMetaList'] = self.get_preloaded_meta_list()
@@ -951,6 +980,19 @@ class KoGridView(BaseFilterView, GridActionsMixin):
             # Initial BaseGridWidget.get_context() / .discover_grid_options() server-side call.
             return self.template_options['pageRouteKwargs'][k]
 
+    def set_grid_fields(self):
+        self.grid_fields_attnames = []
+        self.virtual_fields = set()
+        for field_def in self.yield_fields():
+            if isinstance(field_def, dict):
+                self.grid_fields_attnames.append(field_def['field'])
+                if field_def.get('virtual', False):
+                    self.virtual_fields.add(field_def['field'])
+            elif isinstance(field_def, tuple):
+                self.grid_fields_attnames.append(field_def[0])
+            else:
+                self.grid_fields_attnames.append(field_def)
+
     def setup(self, request, *args, **kwargs):
         super().setup(request, *args, **kwargs)
         if self.query_fields is None:
@@ -961,6 +1003,14 @@ class KoGridView(BaseFilterView, GridActionsMixin):
             self.vm_error('KoGridView.allowed_filter_fields must be instance of OrderedDict')
         return super().get_filters()
 
+    def get_related_fields(self, query_fields=None):
+        query_fields = super().get_related_fields(query_fields)
+        # Remove virtual fields from queryset values().
+        for field_name in self.virtual_fields:
+            if self.has_grid_field(field_name):
+                query_fields.remove(field_name)
+        return query_fields
+
     def set_row_related_fields(self, row):
         row_related = {}
         related_fields = self.get_related_fields()
